@@ -13,6 +13,7 @@ from numpy import (
     prod,
 )
 
+import warnings
 from .BinaryIsometricOp import BinaryIsometricOp
 from .Cast import Cast
 from .Combine import Combine
@@ -798,13 +799,54 @@ class DelayedArray:
                     sanitized.append(dummy)
             return DelayedArray(Subset(self._seed, (*sanitized,)))
 
-        # WHATEVER. Fuck this shit. Just do whatever.
-        test = extract_array(self._seed)[(..., *args)]
+        # If we're discarding dimensions, we see if we can do some pre-emptive extraction.
+        failed = False
+        as_vector = []
+        new_args = []
+        dim_loss = 0
+
+        for d, idx in enumerate(args):
+            if isinstance(idx, ndarray):
+                if len(idx.shape) != 1:
+                    failed = True
+                    break
+            elif isinstance(idx, slice):
+                idx = range(*idx.indices(self.shape[d]))
+            elif not isinstance(idx, Sequence):
+                as_vector.append([idx])
+                new_args.append(0)
+                dim_loss += 1
+                continue
+
+            as_vector.append(idx)
+            new_args.append(slice(None))
+
+        if not failed:
+            # Just using Subset here to avoid having to reproduce the
+            # uniquifying/sorting of subsets before extract_array().
+            base_seed = extract_array(Subset(self._seed, (*as_vector,)))
+        else:
+            base_seed = extract_array(self._seed)
+            new_args = args
+
+        try:
+            test = base_seed[(..., *new_args)]
+            if len(test.shape) != ndim - dim_loss:
+                raise ValueError(
+                    "slicing for "
+                    + str(type(base_seed))
+                    + " does not discard dimensions with scalar indices"
+                )
+        except Exception as e:
+            warnings.warn(e)
+            test = _densify(base_seed)[(..., *new_args)]
+
         if len(test.shape) == ndim:
             raise NotImplementedError(
                 "Oops. Looks like the DelayedArray doesn't correctly handle this combination of index types, but it "
                 "probably should. Consider filing an issue in at https://github.com/BiocPy/DelayedArray/issues."
             )
+
         return test
 
     # For python-level compute.
@@ -813,7 +855,8 @@ class DelayedArray:
         target = extract_array(self._seed)
         try:
             return target.sum(*args, **kwargs)
-        except Exception:
+        except Exception as e:
+            warnings.warn(e)
             target = _densify(target)
             return target.sum(*args, **kwargs)
 
@@ -822,7 +865,8 @@ class DelayedArray:
         target = extract_array(self._seed)
         try:
             return target.var(*args, **kwargs)
-        except Exception:
+        except Exception as e:
+            warnings.warn(e)
             target = _densify(target)
             return target.var(*args, **kwargs)
 
@@ -831,7 +875,8 @@ class DelayedArray:
         target = extract_array(self._seed)
         try:
             return target.mean(*args, **kwargs)
-        except Exception:
+        except Exception as e:
+            warnings.warn(e)
             target = _densify(target)
             return target.mean(*args, **kwargs)
 
