@@ -694,9 +694,18 @@ def test_DelayedArray_subset():
     assert sub.shape == (30, 3, 20)
     assert (numpy.array(sub) == y[:, [1, 3, 5]]).all()
 
+    # Works with unsorted or duplicate indices.
+    sub = x[:, :, [1, 1, 2, 3]]
+    assert (numpy.array(sub) == y[:, :, [1, 1, 2, 3]]).all()
+
+    sub = x[:, [5, 4, 3, 2, 1, 0], :]
+    assert (numpy.array(sub) == y[:, [5, 4, 3, 2, 1, 0], :]).all()
+
     # Falls back to a concrete numpy.ndarray
     stuff = x[:, :, 2]
     assert (stuff == y[:, :, 2]).all()
+    stuff = x[0, :, 2]
+    assert (stuff == y[0, :, 2]).all()
 
     # Works with dask arrays.
     da = delayedarray.create_dask_array(x)
@@ -822,11 +831,18 @@ def test_DelayedArray_sparse():
     out = delayedarray.extract_array(z)
     assert (y.toarray() * v == numpy.array(out)).all()
 
+    z = x / v
+    out = delayedarray.extract_array(z)
+    assert numpy.allclose(y.toarray() / v, out.toarray())
+
     z = x[1:3, [0, 2]]
     out = delayedarray.extract_array(z)
     assert isinstance(out, numpy.ndarray) is False
     assert (y.toarray()[1:3, [0, 2]] == out.toarray()).all()
 
+    assert (y.toarray()[1, :] == x[1, :]).all()
+    assert (y.toarray()[:, 0] == x[:, 0]).all()
+
     z = numpy.log1p(x)
     out = delayedarray.extract_array(z)
     assert isinstance(out, numpy.ndarray) is False
